indexing
	description: "{
      HASH_TABLE with alternate sorted representation.
}"
	license: "Eiffel Forumn License v2 - See license.txt"
	source: "{
		Amalasoft Corporation
		273 Harwood Avenue
		Littleton, MA 01460 USA
		}"
	status: "Copyright 1995-2008, Amalasoft";
	date: "$Date: 2008/12/03 $";
	revision: "$Revision: 001$";
	howto: "{
    Create with an initial capacity.
	 Use as you would a normal HASH_TABLE.
	 As elements change, sorting becomes incoherent and is marked as such.
	 If sorted contents are desired, first check for coherence, then sort
    if necessary.
}"

class AEL_DS_SORTABLE_HASH_TABLE [G->COMPARABLE, H->HASHABLE]

inherit
	HASH_TABLE [G, H]
		export
			{NONE} put, force, replace, merge, copy
		redefine
			extend, remove, clear_all, wipe_out
		end

create
	make

--|========================================================================
feature -- Element change
--|========================================================================

	extend (v: G; k: H) is
			-- Assuming there is no item of key `k',
			-- insert `v' with `k'.
			-- Set `inserted'.
		do
			Precursor (v, k)
			sort_is_coherent := False
		end

	remove (k: H) is
			-- Remove item associated with `k', if present.
			-- Set `removed' if and only if an item has been
			-- removed (i.e. `k' was present);
			-- if so, set `position' to index of removed element.
			-- If not, set `not_found'.
			-- Reset `found_item' to its default value if `removed'.
		do
			Precursor (k)
			sort_is_coherent := False
		end

	clear_all is
			-- Reset all items to default values; reset status.
		do
			Precursor
			sort_is_coherent := False
		end

	wipe_out is
			-- Reset all items to default values; reset status.
		do
			Precursor
			sort_is_coherent := False
		end

--|========================================================================
feature -- Alternate representation
 --|========================================================================

	contents_sorted: AEL_DS_LIST_AGENT_SORTED [like item] is
		-- Contents of this table sorted by item
		do
			if not sort_is_coherent then
				create cached_contents_sorted.make (agent item_compare)
				from start
				until after
				loop
					cached_contents_sorted.extend (item_for_iteration)
					forth
				end
				sort_is_coherent := True
			end
			Result := cached_contents_sorted
		ensure
			exists: Result /= Void
			equivalent: Result.count = count
		end

--|========================================================================
feature {NONE} -- Implementation
--|========================================================================
	
	sort_is_coherent: BOOLEAN
			-- Are the cached sorted contents coherent with the table contents? 

	cached_contents_sorted: like contents_sorted
			-- Contents of this table at last sorting event, sorted by item

	--|--------------------------------------------------------------

	item_compare (v1, v2: like item): BOOLEAN is
			-- Is v1 less than v2?
		do
			Result := v1 < v2
		end

end -- class AEL_DS_SORTABLE_HASH_TABLE
